@@ -3,6 +3,8 @@
  * Copyright (c) 2008-2013	Laurent Destailleur		<eldy@users.sourceforge.net>
  * Copyright (C) 2012		Regis Houssin			<regis.houssin@inodbox.com>
  * Copyright (C) 2012       Marcos García           <marcosgdf@gmail.com>
+ * Copyright (C) 2024		MDW						<mdeweerd@users.noreply.github.com>
+ * Copyright (C) 2024       Frédéric France         <frederic.france@free.fr>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,9 +31,19 @@
  */
 abstract class Stats
 {
+	/**
+	 * @var DoliDB
+	 */
 	protected $db;
-	protected $lastfetchdate = array(); // Dates of cache file read by methods
-	public $cachefilesuffix = ''; // Suffix to add to name of cache file (to avoid file name conflicts)
+	/**
+	 * @var array<string,int>	Dates of cache file read by methods
+	 */
+	protected $lastfetchdate = array();
+
+	/**
+	 * @var string  Suffix to add to name of cache file (to avoid file name conflicts)
+	 */
+	public $cachefilesuffix = '';
 
 	/**
 	 * @var string	To store the FROM part of the main table of the SQL request
@@ -75,21 +87,21 @@ abstract class Stats
 	public $month;
 
 	/**
-	 *  @param	int		$year 			number
-	 * 	@param	int 	$format 		0=Label of abscissa is a translated text, 1=Label of abscissa is month number, 2=Label of abscissa is first letter of month
-	 * 	@return int						value
+	 *	@param	int		$year 			number
+	 *	@param	int 	$format 		0=Label of abscissa is a translated text, 1=Label of abscissa is month number, 2=Label of abscissa is first letter of month
+	 *	@return	array<int<0,11>,array{0:int<1,12>,1:int}>	Array of nb each month
 	 */
 	abstract protected function getNbByMonth($year, $format = 0);
 
 	/**
 	 * Return nb of elements by month for several years
 	 *
-	 * @param 	int		$endyear		Start year
-	 * @param 	int		$startyear		End year
-	 * @param	int		$cachedelay		Delay we accept for cache file (0=No read, no save of cache, -1=No read but save)
-	 * @param	int		$format			0=Label of abscissa is a translated text, 1=Label of abscissa is month number, 2=Label of abscissa is first letter of month
-	 * @param   int 	$startmonth		month of the fiscal year start min 1 max 12 ; if 1 = january
-	 * @return 	array					Array of values
+	 *	@param 	int		$endyear		Start year
+	 *	@param 	int		$startyear		End year
+	 *	@param	int		$cachedelay		Delay we accept for cache file (0=No read, no save of cache, -1=No read but save)
+	 *	@param	int		$format			0=Label of abscissa is a translated text, 1=Label of abscissa is month number, 2=Label of abscissa is first letter of month
+	 *	@param   int 	$startmonth		month of the fiscal year start min 1 max 12 ; if 1 = january
+	 * @return	array<int<0,11>,array{0:int<1,12>,1:int}>	Array of values
 	 */
 	public function getNbByMonthWithPrevYear($endyear, $startyear, $cachedelay = 0, $format = 0, $startmonth = 1)
 	{
@@ -99,12 +111,12 @@ abstract class Stats
 			return array();
 		}
 
-		$datay = array();
+		$data  = array();  // This is the return value
+		$datay = array();  // This is a work value
 
 		// Search into cache
 		if (!empty($cachedelay)) {
 			include_once DOL_DOCUMENT_ROOT.'/core/lib/files.lib.php';
-			include_once DOL_DOCUMENT_ROOT.'/core/lib/json.lib.php';
 		}
 
 		$newpathofdestfile = $conf->user->dir_temp.'/'.get_class($this).'_'.__FUNCTION__.'_'.(empty($this->cachefilesuffix) ? '' : $this->cachefilesuffix.'_').$langs->defaultlang.'_entity.'.$conf->entity.'_user'.$user->id.'.cache';
@@ -113,6 +125,7 @@ abstract class Stats
 		$nowgmt = dol_now();
 
 		$foundintocache = 0;
+		$filedate = -1;
 		if ($cachedelay > 0) {
 			$filedate = dol_filemtime($newpathofdestfile);
 			if ($filedate >= ($nowgmt - $cachedelay)) {
@@ -127,62 +140,61 @@ abstract class Stats
 		if ($foundintocache) {    // Cache file found and is not too old
 			dol_syslog(get_class($this).'::'.__FUNCTION__." read data from cache file ".$newpathofdestfile." ".$filedate.".");
 			$data = json_decode(file_get_contents($newpathofdestfile), true);
+			'@phan-var-force array<int<0,11>,array{0:int<1,12>,1:int}> $data';
 		} else {
 			$year = $startyear;
 			$sm = $startmonth - 1;
 			if ($sm != 0) {
-				$year = $year - 1;
+				$year -= 1;
 			}
 			while ($year <= $endyear) {
 				$datay[$year] = $this->getNbByMonth($year, $format);
 				$year++;
 			}
 
-			$data = array();
-
 			for ($i = 0; $i < 12; $i++) {
 				$data[$i][] = $datay[$endyear][($i + $sm) % 12][0];
 				$year = $startyear;
 				while ($year <= $endyear) {
 					// floor(($i + $sm) / 12)) is 0 if we are after the month start $sm and same year, become 1 when we reach january of next year
-					$data[$i][] = $datay[$year - (1 - floor(($i + $sm) / 12)) + ($sm == 0 ? 1 : 0)][($i + $sm) % 12][1];
+					$data[$i][] = $datay[$year - (1 - (int) floor(($i + $sm) / 12)) + ($sm == 0 ? 1 : 0)][($i + $sm) % 12][1];
 					$year++;
 				}
 			}
 		}
 
+
 		// Save cache file
 		if (empty($foundintocache) && ($cachedelay > 0 || $cachedelay == -1)) {
 			dol_syslog(get_class($this).'::'.__FUNCTION__." save cache file ".$newpathofdestfile." onto disk.");
 			if (!dol_is_dir($conf->user->dir_temp)) {
 				dol_mkdir($conf->user->dir_temp);
 			}
-			$fp = fopen($newpathofdestfile, 'w');
+			$fp = @fopen($newpathofdestfile, 'w');
 			if ($fp) {
 				fwrite($fp, json_encode($data));
 				fclose($fp);
 			} else {
-				dol_syslog("Failed to save cache file ".$newpathofdestfile);
+				dol_syslog("Failed to save cache file ".$newpathofdestfile, LOG_ERR);
 			}
 			dolChmod($newpathofdestfile);
 
 			$this->lastfetchdate[get_class($this).'_'.__FUNCTION__] = $nowgmt;
 		}
 
-		// return array(array('Month',val1,val2,val3),...)
 		return $data;
 	}
 
 	/**
-	 * @param	int		$year			year number
-	 * @param	int 	$format			0=Label of abscissa is a translated text, 1=Label of abscissa is month number, 2=Label of abscissa is first letter of month
-	 * @return 	int						value
+	 *	@param	int		$year			year number
+	 *	@param	int 	$format			0=Label of abscissa is a translated text, 1=Label of abscissa is month number, 2=Label of abscissa is first letter of month
+	 *	@return	array<int<0,11>,array{0:int<1,12>,1:int|float}>	Array of values by month
 	 */
 	abstract protected function getAmountByMonth($year, $format = 0);
 
 	/**
 	 * Return amount of elements by month for several years.
-	 * Criterias used to build request are defined into the constructor of parent class into xxx/class/xxxstats.class.php
+	 * Criteria used to build request are defined into the constructor of parent class into xxx/class/xxxstats.class.php
 	 * The caller of class can add more filters into sql request by adding criteris into the $stats->where property just after
 	 * calling constructor.
 	 *
@@ -191,7 +203,7 @@ abstract class Stats
 	 * @param	int		$cachedelay		Delay we accept for cache file (0=No read, no save of cache, -1=No read but save)
 	 * @param	int		$format			0=Label of abscissa is a translated text, 1=Label of abscissa is month number, 2=Label of abscissa is first letter of month
 	 * @param   int 	$startmonth		month of the fiscal year start min 1 max 12 ; if 1 = january
-	 * @return 	array					Array of values
+	 * @return	array<int<0,11>,array<string|int|float>>	Array of values
 	 */
 	public function getAmountByMonthWithPrevYear($endyear, $startyear, $cachedelay = 0, $format = 0, $startmonth = 1)
 	{
@@ -202,11 +214,11 @@ abstract class Stats
 		}
 
 		$datay = array();
+		$data = array();  // Return value
 
 		// Search into cache
 		if (!empty($cachedelay)) {
 			include_once DOL_DOCUMENT_ROOT.'/core/lib/files.lib.php';
-			include_once DOL_DOCUMENT_ROOT.'/core/lib/json.lib.php';
 		}
 
 		$newpathofdestfile = $conf->user->dir_temp.'/'.get_class($this).'_'.__FUNCTION__.'_'.(empty($this->cachefilesuffix) ? '' : $this->cachefilesuffix.'_').$langs->defaultlang.'_entity.'.$conf->entity.'_user'.$user->id.'.cache';
@@ -215,6 +227,7 @@ abstract class Stats
 		$nowgmt = dol_now();
 
 		$foundintocache = 0;
+		$filedate = -1;
 		if ($cachedelay > 0) {
 			$filedate = dol_filemtime($newpathofdestfile);
 			if ($filedate >= ($nowgmt - $cachedelay)) {
@@ -234,21 +247,20 @@ abstract class Stats
 			$year = $startyear;
 			$sm = $startmonth - 1;
 			if ($sm != 0) {
-				$year = $year - 1;
+				$year -= 1;
 			}
 			while ($year <= $endyear) {
 				$datay[$year] = $this->getAmountByMonth($year, $format);
 				$year++;
 			}
 
-			$data = array();
 			// $data = array('xval'=>array(0=>xlabel,1=>yval1,2=>yval2...),...)
 			for ($i = 0; $i < 12; $i++) {
 				$data[$i][] = isset($datay[$endyear][($i + $sm) % 12]['label']) ? $datay[$endyear][($i + $sm) % 12]['label'] : $datay[$endyear][($i + $sm) % 12][0]; // set label
 				$year = $startyear;
 				while ($year <= $endyear) {
 					// floor(($i + $sm) / 12)) is 0 if we are after the month start $sm and same year, become 1 when we reach january of next year
-					$data[$i][] = $datay[$year - (1 - floor(($i + $sm) / 12)) + ($sm == 0 ? 1 : 0)][($i + $sm) % 12][1]; // set yval for x=i
+					$data[$i][] = $datay[$year - (1 - (int) floor(($i + $sm) / 12)) + ($sm == 0 ? 1 : 0)][($i + $sm) % 12][1]; // set yval for x=i
 					$year++;
 				}
 			}
@@ -260,14 +272,15 @@ abstract class Stats
 			if (!dol_is_dir($conf->user->dir_temp)) {
 				dol_mkdir($conf->user->dir_temp);
 			}
-			$fp = fopen($newpathofdestfile, 'w');
+			$fp = @fopen($newpathofdestfile, 'w');
 			if ($fp) {
 				fwrite($fp, json_encode($data));
 				fclose($fp);
-				dolChmod($newpathofdestfile);
 			} else {
-				dol_syslog("Failed to write cache file", LOG_ERR);
+				dol_syslog("Failed to save cache file ".$newpathofdestfile, LOG_ERR);
 			}
+			dolChmod($newpathofdestfile);
+
 			$this->lastfetchdate[get_class($this).'_'.__FUNCTION__] = $nowgmt;
 		}
 
@@ -275,17 +288,17 @@ abstract class Stats
 	}
 
 	/**
-	 * @param	int     $year           year number
-	 * @return 	array					array of values
+	 *	@param	int     $year           year number
+	 *	@return	array<int<0,11>,array{0:int<1,12>,1:int|float}>	Array of values
 	 */
 	abstract protected function getAverageByMonth($year);
 
 	/**
-	 * Return average of entity by month for several years
+	 *	Return average of entity by month for several years
 	 *
-	 * @param	int		$endyear		Start year
-	 * @param	int		$startyear		End year
-	 * @return 	array					Array of values
+	 *	@param	int		$endyear		Start year
+	 *	@param	int		$startyear		End year
+	 *	@return array<array<int|float>>	Array of values
 	 */
 	public function getAverageByMonthWithPrevYear($endyear, $startyear)
 	{
@@ -294,6 +307,7 @@ abstract class Stats
 		}
 
 		$datay = array();
+		$data = array();
 
 		$year = $startyear;
 		while ($year <= $endyear) {
@@ -301,8 +315,6 @@ abstract class Stats
 			$year++;
 		}
 
-		$data = array();
-
 		for ($i = 0; $i < 12; $i++) {
 			$data[$i][] = $datay[$endyear][$i][0];
 			$year = $startyear;
@@ -316,12 +328,12 @@ abstract class Stats
 	}
 
 	/**
-	 * Return count, and sum of products
+	 *	Return count, and sum of products
 	 *
-	 * @param	int		$year			Year
-	 * @param	int		$cachedelay		Delay we accept for cache file (0=No read, no save of cache, -1=No read but save)
-	 * @param  	int     $limit      	Limit
-	 * @return 	array					Array of values
+	 *	@param	int		$year			Year
+	 *	@param	int		$cachedelay		Delay we accept for cache file (0=No read, no save of cache, -1=No read but save)
+	 *	@param 	int     $limit      	Limit
+	 *	@return	array<int<0,11>,array{0:int<1,12>,1:int|float}>	Array of values
 	 */
 	public function getAllByProductEntry($year, $cachedelay = 0, $limit = 10)
 	{
@@ -329,10 +341,9 @@ abstract class Stats
 
 		$data = array();
 
-		// Search into cache
+		// Search in cache
 		if (!empty($cachedelay)) {
 			include_once DOL_DOCUMENT_ROOT.'/core/lib/files.lib.php';
-			include_once DOL_DOCUMENT_ROOT.'/core/lib/json.lib.php';
 		}
 
 		$newpathofdestfile = $conf->user->dir_temp.'/'.get_class($this).'_'.__FUNCTION__.'_'.(empty($this->cachefilesuffix) ? '' : $this->cachefilesuffix.'_').$langs->defaultlang.'_entity.'.$conf->entity.'_user'.$user->id.'.cache';
@@ -341,6 +352,7 @@ abstract class Stats
 		$nowgmt = dol_now();
 
 		$foundintocache = 0;
+		$filedate = -1;
 		if ($cachedelay > 0) {
 			$filedate = dol_filemtime($newpathofdestfile);
 			if ($filedate >= ($nowgmt - $cachedelay)) {
@@ -356,9 +368,8 @@ abstract class Stats
 		if ($foundintocache) {    // Cache file found and is not too old
 			dol_syslog(get_class($this).'::'.__FUNCTION__." read data from cache file ".$newpathofdestfile." ".$filedate.".");
 			$data = json_decode(file_get_contents($newpathofdestfile), true);
+			'@phan-var-force array<int<0,11>,array{0:int<1,12>,1:int|float}> $data';  // Phan can't decode json_decode's return value
 		} else {
-			// This method is defined in parent object only, not into abstract, so we disable phpstan warning
-			/** @phpstan-ignore-next-line */
 			$data = $this->getAllByProduct($year, $limit);
 		}
 
@@ -368,12 +379,15 @@ abstract class Stats
 			if (!dol_is_dir($conf->user->dir_temp)) {
 				dol_mkdir($conf->user->dir_temp);
 			}
-			$fp = fopen($newpathofdestfile, 'w');
+			$fp = @fopen($newpathofdestfile, 'w');
 			if ($fp) {
 				fwrite($fp, json_encode($data));
 				fclose($fp);
-				dolChmod($newpathofdestfile);
+			} else {
+				dol_syslog("Failed to save cache file ".$newpathofdestfile, LOG_ERR);
 			}
+			dolChmod($newpathofdestfile);
+
 			$this->lastfetchdate[get_class($this).'_'.__FUNCTION__] = $nowgmt;
 		}
 
@@ -389,7 +403,7 @@ abstract class Stats
 	 * 	Return nb of elements by year
 	 *
 	 *	@param	string	$sql		SQL request
-	 * 	@return	array
+	 * @return	array<array{0:int,1:int}>				Array of nb each year
 	 */
 	protected function _getNbByYear($sql)
 	{
@@ -418,7 +432,7 @@ abstract class Stats
 	 * 	Return nb of elements, total amount and avg amount each year
 	 *
 	 *	@param	string	$sql	SQL request
-	 * 	@return	array			Array with nb, total amount, average for each year
+	 * 	@return	array<array{year:string,nb:string,nb_diff:float,total?:float,avg?:float,weighted?:float,total_diff?:float,avg_diff?:float,avg_weighted?:float}>	Array with nb, total amount, average for each year
 	 */
 	protected function _getAllByYear($sql)
 	{
@@ -469,13 +483,13 @@ abstract class Stats
 
 	// phpcs:disable PEAR.NamingConventions.ValidFunctionName.PublicUnderscore
 	/**
-	 *     Renvoie le nombre de documents par mois pour une annee donnee
-	 *     Return number of documents per month for a given year
+	 *	Renvoie le nombre de documents par mois pour une annee donnee
+	 *	Return number of documents per month for a given year
 	 *
-	 *     @param   int		$year       Year
-	 *     @param   string	$sql        SQL
-	 *     @param	int		$format		0=Label of abscissa is a translated text, 1=Label of abscissa is month number, 2=Label of abscissa is first letter of month
-	 *     @return	array				Array of nb each month
+	 *	@param	int			$year       Year
+	 *	@param	string		$sql        SQL
+	 *	@param	int<0,2>	$format		0=Label of abscissa is a translated text, 1=Label of abscissa is month number, 2=Label of abscissa is first letter of month
+	 *	@return	array<int<0,11>,array{0:int<1,12>,1:int}>	Array of nb each month
 	 */
 	protected function _getNbByMonth($year, $sql, $format = 0)
 	{
@@ -527,12 +541,12 @@ abstract class Stats
 
 	// phpcs:disable PEAR.NamingConventions.ValidFunctionName.PublicUnderscore
 	/**
-	 *     Return the amount per month for a given year
+	 *	Return the amount per month for a given year
 	 *
-	 *     @param	int		$year       Year
-	 *     @param   string	$sql		SQL
-	 *     @param	int		$format		0=Label of abscissa is a translated text, 1=Label of abscissa is month number, 2=Label of abscissa is first letter of month
-	 *     @return	array
+	 *	@param	int			$year       Year
+	 *	@param	string		$sql		SQL
+	 *	@param	int<0,2>	$format		0=Label of abscissa is a translated text, 1=Label of abscissa is month number, 2=Label of abscissa is first letter of month
+	 *	@return	array<int<0,11>,array{0:int<1,12>,1:int|float}>	Array of nb each month
 	 */
 	protected function _getAmountByMonth($year, $sql, $format = 0)
 	{
@@ -584,13 +598,12 @@ abstract class Stats
 
 	// phpcs:disable PEAR.NamingConventions.ValidFunctionName.PublicUnderscore
 	/**
-	 *  Renvoie le montant moyen par mois pour une annee donnee
 	 *  Return the amount average par month for a given year
 	 *
-	 *  @param  int     $year       Year
-	 *  @param  string  $sql        SQL
-	 *  @param  int     $format     0=Label of abscissa is a translated text, 1=Label of abscissa is month number, 2=Label of abscissa is first letter of month
-	 *  @return array
+	 *  @param  int			$year       Year
+	 *  @param  string		$sql        SQL
+	 *  @param  int<0,2>	$format     0=Label of abscissa is a translated text, 1=Label of abscissa is month number, 2=Label of abscissa is first letter of month
+	 *	@return	array<int<0,11>,array{0:int<1,12>,1:int|float}>	Array of average each month
 	 */
 	protected function _getAverageByMonth($year, $sql, $format = 0)
 	{
@@ -647,7 +660,7 @@ abstract class Stats
 	 *
 	 *  @param  string  $sql        SQL
 	 *  @param  int     $limit      Limit
-	 *  @return array
+	 *	@return	array<int<0,11>,array{0:int<1,12>,1:int|float}>	Array of total of product refs each month
 	 */
 	protected function _getAllByProduct($sql, $limit = 10)
 	{
@@ -686,7 +699,7 @@ abstract class Stats
 	/**
 	 *  Returns the summed amounts per year for a given number of past years ending now
 	 *  @param  string  $sql    SQL
-	 *  @return array
+	 *	@return	array<int,array{0:int,1:int}>	Array of sum of amounts
 	 */
 	protected function _getAmountByYear($sql)
 	{
@@ -697,7 +710,6 @@ abstract class Stats
 			$i = 0;
 			while ($i < $num) {
 				$row = $this->db->fetch_row($resql);
-				$j = (int) $row[0];
 				$result[] = [
 					0 => (int) $row[0],
 					1 => (int) $row[1],
@@ -708,4 +720,21 @@ abstract class Stats
 		}
 		return $result;
 	}
+
+
+	/**
+	 *	Return nb, amount of predefined product for year
+	 *
+	 *	@param	int		$year		Year to scan
+	 *  @param  int     $limit      Limit
+	 *	@return	array<int<0,11>,array{0:int<1,12>,1:int|float}>	Array of values
+	 */
+	public function getAllByProduct($year, $limit = 0)
+	{
+		// Needs to be implemented in child class when used
+		$msg = get_class($this)."::".__FUNCTION__." not implemented";
+		dol_syslog($msg, LOG_ERR);
+		$l = array(1,0); // Dummy result
+		return array($l,$l,$l,$l,$l,$l,$l,$l,$l,$l,$l,$l);
+	}
 }
