@@ -2,6 +2,8 @@
 /*
  * Copyright (C) 2013-2015 Raphaël Doursenaud <rdoursenaud@gpcsolutions.fr>
  * Copyright (C) 2014-2015 Laurent Destailleur <eldy@users.sourceforge.net>
+ * Copyright (C) 2024		MDW							<mdeweerd@users.noreply.github.com>
+ * Copyright (C) 2024       Frédéric France             <frederic.france@free.fr>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,12 +26,18 @@
 
 require_once DOL_DOCUMENT_ROOT.'/core/db/Database.interface.php';
 
+
 /**
  * Class to manage Dolibarr database access
  */
 abstract class DoliDB implements Database
 {
-	/** @var bool|resource|mysqli|SQLite3|PgSql\Connection Database handler */
+	/** Force subclass to implement VERSIONMIN - required DB version */
+	const VERSIONMIN = self::VERSIONMIN;
+	/** Force subclass to implement LABEL - description of DB type */
+	const LABEL = self::LABEL;
+
+	/** @var false|resource|mysqli|mysqliDoli|SQLite3|PgSql\Connection|DoliDB Database handler */
 	public $db;
 	/** @var string Database type */
 	public $type;
@@ -39,7 +47,7 @@ abstract class DoliDB implements Database
 	public $forcecollate = 'utf8_unicode_ci';
 
 	/** @var resource Resultset of last query */
-	private $_results;
+	private $_results; // @phpstan-ignore-line
 
 	/** @var bool true if connected, else false */
 	public $connected;
@@ -89,8 +97,8 @@ abstract class DoliDB implements Database
 	 *	Format a SQL IF
 	 *
 	 *	@param	string	$test           Test string (example: 'cd.statut=0', 'field IS NULL')
-	 *	@param	string	$resok          resultat si test egal
-	 *	@param	string	$resko          resultat si test non egal
+	 *	@param	string	$resok          resultat si test equal
+	 *	@param	string	$resko          resultat si test non equal
 	 *	@return	string          		SQL string
 	 */
 	public function ifsql($test, $resok, $resko)
@@ -99,13 +107,25 @@ abstract class DoliDB implements Database
 		return '(CASE WHEN '.$test.' THEN '.$resok.' ELSE '.$resko.' END)';
 	}
 
+	/**
+	 * Return SQL string to aggregate using the Standard Deviation of population
+	 *
+	 * @param	string	$nameoffield	Name of field
+	 * @return	string					SQL string
+	 */
+	public function stddevpop($nameoffield)
+	{
+		return 'STDDEV_POP('.$nameoffield.')';
+	}
+
 	/**
 	 * Return SQL string to force an index
 	 *
 	 * @param	string	$nameofindex	Name of index
+	 * @param	int		$mode			0=Use, 1=Force
 	 * @return	string					SQL string
 	 */
-	public function hintindex($nameofindex)
+	public function hintindex($nameofindex, $mode = 1)
 	{
 		return '';
 	}
@@ -114,18 +134,18 @@ abstract class DoliDB implements Database
 	/**
 	 *	Format a SQL REGEXP
 	 *
-	 *	@param	string	$subject        string tested
+	 *	@param	string	$subject        Field name to test
 	 *	@param	string  $pattern        SQL pattern to match
-	 *	@param	int		$sqlstring      whether or not the string being tested is an SQL expression
+	 *	@param	int		$sqlstring      0=the string being tested is a hard coded string, 1=the string is a field
 	 *	@return	string          		SQL string
 	 */
 	public function regexpsql($subject, $pattern, $sqlstring = 0)
 	{
 		if ($sqlstring) {
-			return "(". $subject ." REGEXP '" . $pattern . "')";
+			return "(". $subject ." REGEXP '" . $this->escape($pattern) . "')";
 		}
 
-		return "('". $subject ."' REGEXP '" . $pattern . "')";
+		return "('". $this->escape($subject) ."' REGEXP '" . $this->escape($pattern) . "')";
 	}
 
 
@@ -134,7 +154,7 @@ abstract class DoliDB implements Database
 	 *   Function to use to build INSERT, UPDATE or WHERE predica
 	 *
 	 *   @param	    int		$param      Date TMS to convert
-	 *	 @param		mixed	$gm			'gmt'=Input informations are GMT values, 'tzserver'=Local to server TZ
+	 *	 @param		'gmt'|'tzserver'	$gm		'gmt'=Input information are GMT values, 'tzserver'=Local to server TZ
 	 *   @return	string      		Date in a string YYYY-MM-DD HH:MM:SS
 	 */
 	public function idate($param, $gm = 'tzserver')
@@ -156,22 +176,42 @@ abstract class DoliDB implements Database
 	/**
 	 * Sanitize a string for SQL forging
 	 *
-	 * @param   string 	$stringtosanitize 	String to escape
-	 * @param   int		$allowsimplequote 	1=Allow simple quotes in string. When string is used as a list of SQL string ('aa', 'bb', ...)
+	 * @param   string 	$stringtosanitize 	String to sanitize
+	 * @param   int		$allowsimplequote 	1=Allow simple quotes in string around val separated by "," but only when string is used as a list of SQL string "'aa', 'bb', 'cc', ..."). Can be used for IN ...
+	 * 										2=Allow all simple quotes. If you use this value, the return MUST be escaped to forge SQL strings.
 	 * @param	int		$allowsequals		1=Allow equals sign
 	 * @param	int		$allowsspace		1=Allow space char
+	 * @param	int		$allowschars		1=Allow a-z chars
 	 * @return  string                      String escaped
 	 */
-	public function sanitize($stringtosanitize, $allowsimplequote = 0, $allowsequals = 0, $allowsspace = 0)
+	public function sanitize($stringtosanitize, $allowsimplequote = 0, $allowsequals = 0, $allowsspace = 0, $allowschars = 1)
 	{
-		return preg_replace('/[^a-z0-9_\-\.,'.($allowsequals ? '=' : '').($allowsimplequote ? "\'" : '').($allowsspace ? ' ' : '').']/i', '', $stringtosanitize);
+		$result = preg_replace('/[^0-9_\-\.,'.($allowschars ? '\p{L}' : '').($allowsequals ? '=' : '').($allowsimplequote ? "\'" : '').($allowsspace ? ' ' : '').']/ui', '', $stringtosanitize);
+		//$result = preg_replace('/[^0-9_\-\.,'.($allowschars ? 'a-z' : '').($allowsequals ? '=' : '').($allowsimplequote ? "\'" : '').($allowsspace ? ' ' : '').']/i', '', $stringtosanitize);
+
+		if ($allowsimplequote == 1) {
+			// Remove all quotes that are inside a string and not around
+			$tmpchars = explode(',', $result);
+			$newstringarray = array();
+			foreach ($tmpchars as $tmpchar) {
+				$reg = array();
+				if (preg_match('/^\'(.*)\'$/', $tmpchar, $reg)) {
+					$newstringarray[] = "'".str_replace("'", "", $reg[1])."'";
+				} else {
+					$newstringarray[] = str_replace("'", "", $tmpchar);
+				}
+			}
+			$result = implode(',', $newstringarray);
+		}
+
+		return $result;
 	}
 
 	/**
 	 * Start transaction
 	 *
 	 * @param	string	$textinlog		Add a small text into log. '' by default.
-	 * @return	int         			1 if transaction successfuly opened or already opened, 0 if error
+	 * @return	int         			1 if transaction successfully opened or already opened, 0 if error
 	 */
 	public function begin($textinlog = '')
 	{
@@ -220,7 +260,7 @@ abstract class DoliDB implements Database
 	 *	Cancel a transaction and go back to initial data values
 	 *
 	 * 	@param	string			$log		Add more log to default log line
-	 * 	@return	resource|int         		1 if cancelation is ok or transaction not open, 0 if error
+	 * 	@return	resource|int         		1 if cancellation is ok or transaction not open, 0 if error
 	 */
 	public function rollback($log = '')
 	{
@@ -262,7 +302,7 @@ abstract class DoliDB implements Database
 	/**
 	 *	Return version of database server into an array
 	 *
-	 *	@return	        array  		Version array
+	 *	@return	        string[]  		Version array
 	 */
 	public function getVersionArray()
 	{
@@ -283,7 +323,7 @@ abstract class DoliDB implements Database
 	 * Define sort criteria of request
 	 *
 	 * @param	string		$sortfield		List of sort fields, separated by comma. Example: 't1.fielda,t2.fieldb'
-	 * @param	string		$sortorder		Sort order, separated by comma. Example: 'ASC,DESC'. Note: If the quantity fo sortorder values is lower than sortfield, we used the last value for missing values.
+	 * @param	string		$sortorder		Sort order, separated by comma. Example: 'ASC,DESC'. Note: If the quantity for sortorder values is lower than sortfield, we used the last value for missing values.
 	 * @return	string						String to provide syntax of a sort sql string
 	 */
 	public function order($sortfield = '', $sortorder = '')
@@ -292,7 +332,7 @@ abstract class DoliDB implements Database
 			$oldsortorder = '';
 			$return = '';
 			$fields = explode(',', $sortfield);
-			$orders = explode(',', $sortorder);
+			$orders = (!empty($sortorder) ? explode(',', $sortorder) : array());
 			$i = 0;
 			foreach ($fields as $val) {
 				if (!$return) {
@@ -337,11 +377,11 @@ abstract class DoliDB implements Database
 	/**
 	 *	Convert (by PHP) a PHP server TZ string date into a Timestamps date (GMT if gm=true)
 	 * 	19700101020000 -> 3600 with server TZ = +1 and $gm='tzserver'
-	 * 	19700101020000 -> 7200 whaterver is server TZ if $gm='gmt'
+	 * 	19700101020000 -> 7200 whatever is server TZ if $gm='gmt'
 	 *
 	 * 	@param	string				$string		Date in a string (YYYYMMDDHHMMSS, YYYYMMDD, YYYY-MM-DD HH:MM:SS)
-	 *	@param	mixed				$gm			'gmt'=Input informations are GMT values, 'tzserver'=Local to server TZ
-	 *	@return	int|string						Date TMS or ''
+	 *	@param	mixed				$gm			'gmt'=Input information are GMT values, 'tzserver'=Local to server TZ
+	 *	@return	int|''							Date TMS or ''
 	 */
 	public function jdate($string, $gm = 'tzserver')
 	{
@@ -368,7 +408,7 @@ abstract class DoliDB implements Database
 	/**
 	 * Return first result from query as object
 	 * Note : This method executes a given SQL query and retrieves the first row of results as an object. It should only be used with SELECT queries
-	 * Dont add LIMIT to your query, it will be added by this method
+	 * Don't add LIMIT to your query, it will be added by this method
 	 *
 	 * @param 	string 				$sql 	The sql query string
 	 * @return 	bool|int|object    			False on failure, 0 on empty, object on success
@@ -377,10 +417,11 @@ abstract class DoliDB implements Database
 	{
 		$sql .= ' LIMIT 1';
 
-		$res = $this->query($sql);
-		if ($res) {
-			$obj = $this->fetch_object($res);
+		$resql = $this->query($sql);
+		if ($resql) {
+			$obj = $this->fetch_object($resql);
 			if ($obj) {
+				$this->free($resql);
 				return $obj;
 			} else {
 				return 0;
@@ -391,24 +432,28 @@ abstract class DoliDB implements Database
 	}
 
 	/**
-	 * Return all results from query as an array of objects
-	 * Note : This method executes a given SQL query and retrieves all row of results as an array of objects. It should only be used with SELECT queries
-	 * be carefull with this method use it only with some limit of results to avoid performences loss.
+	 * Return all results from query as an array of objects. Using this is a bad practice and is discouraged.
+	 * Note : It should only be used with SELECT queries and with a limit. If you are not able to defined/know what can be the limit, it
+	 * just means this function is not what you need. Do not use it.
 	 *
-	 * @param 	string 		$sql 		The sql query string
-	 * @return 	bool|array				Result
-	 * @deprecated
+	 * @param 	string 			$sql 	The sql query string. Must end with "... LIMIT x"
+	 * @return  false|Object[]          Result
 	 */
 	public function getRows($sql)
 	{
-		$res = $this->query($sql);
-		if ($res) {
+		if (!preg_match('/LIMIT \d+(?:(?:,\ *\d*)|(?:\ +OFFSET\ +\d*))?\ *;?$/', $sql)) {
+			trigger_error(__CLASS__ .'::'.__FUNCTION__.'() query must have a LIMIT clause', E_USER_ERROR);
+		}
+
+		$resql = $this->query($sql);
+		if ($resql) {
 			$results = array();
-			if ($this->num_rows($res) > 0) {
-				while ($obj = $this->fetch_object($res)) {
+			if ($this->num_rows($resql) > 0) {
+				while ($obj = $this->fetch_object($resql)) {
 					$results[] = $obj;
 				}
 			}
+			$this->free($resql);
 			return $results;
 		}
 
