@@ -2,7 +2,9 @@
 /* Copyright (C) 2012       Nicolas Villa aka Boyquotes http://informetic.fr
  * Copyright (C) 2013       Florian Henry           <florian.henry@open-concpt.pro>
  * Copyright (C) 2013-2016  Laurent Destailleur     <eldy@users.sourceforge.net>
- * Copyright (C) 2018-2021  Frédéric France         <frederic.france@netlogic.fr>
+ * Copyright (C) 2018-2024	Frédéric France         <frederic.france@free.fr>
+ * Copyright (C) 2024		William Mead			<william.mead@manchenumerique.fr>
+ * Copyright (C) 2025		MDW						<mdeweerd@users.noreply.github.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,15 +31,23 @@ require '../main.inc.php';
 
 require_once DOL_DOCUMENT_ROOT.'/core/class/doleditor.class.php';
 
-// librairie jobs
+// Cron job libraries
 require_once DOL_DOCUMENT_ROOT."/cron/class/cronjob.class.php";
 require_once DOL_DOCUMENT_ROOT."/core/class/html.formcron.class.php";
 require_once DOL_DOCUMENT_ROOT.'/core/lib/cron.lib.php';
 
+/**
+ * @var Conf $conf
+ * @var DoliDB $db
+ * @var HookManager $hookmanager
+ * @var Translate $langs
+ * @var User $user
+ */
+
 // Load translation files required by the page
 $langs->loadLangs(array('admin', 'cron', 'members', 'bills'));
 
-$id = GETPOST('id', 'int');
+$id = GETPOSTINT('id');
 $action = GETPOST('action', 'aZ09');
 $confirm = GETPOST('confirm', 'alpha');
 $cancel = GETPOST('cancel', 'alpha');
@@ -46,12 +56,14 @@ $backtopageforcancel = GETPOST('backtopageforcancel', 'alpha');
 
 $securitykey = GETPOST('securitykey', 'alpha');
 
-$permissiontoadd = $user->rights->cron->create;
-
-if (!$user->rights->cron->create) {
+if (!$user->hasRight('cron', 'create')) {
 	accessforbidden();
 }
 
+$permissiontoadd = $user->hasRight('cron', 'create');
+$permissiontoexecute = $user->hasRight('cron', 'execute');
+$permissiontodelete = $user->hasRight('cron', 'delete');
+
 
 /*
  * Actions
@@ -71,32 +83,30 @@ if (!empty($cancel)) {
 	} else {
 		if ($backtopage) {
 			header("Location: ".$backtopage);
-			exit;
 		} else {
 			header("Location: ".DOL_URL_ROOT.'/cron/list.php');
-			exit;
 		}
+		exit;
 	}
 }
 
 // Delete jobs
-if ($action == 'confirm_delete' && $confirm == "yes" && $user->rights->cron->delete) {
+if ($action == 'confirm_delete' && $confirm == "yes" && $permissiontodelete) {
 	$result = $object->delete($user);
 
 	if ($result < 0) {
 		setEventMessages($object->error, $object->errors, 'errors');
 		$action = 'edit';
 	} else {
-		Header("Location: ".DOL_URL_ROOT.'/cron/list.php');
+		header("Location: ".DOL_URL_ROOT.'/cron/list.php');
 		exit;
 	}
 }
 
 // Execute jobs
-if ($action == 'confirm_execute' && $confirm == "yes" && $user->rights->cron->execute) {
-	if (!empty($conf->global->CRON_KEY) && $conf->global->CRON_KEY != $securitykey) {
+if ($action == 'confirm_execute' && $confirm == "yes" && $permissiontoexecute) {
+	if (getDolGlobalString('CRON_KEY') && $conf->global->CRON_KEY != $securitykey) {
 		setEventMessages('Security key '.$securitykey.' is wrong', null, 'errors');
-		$action = '';
 	} else {
 		$now = dol_now(); // Date we start
 
@@ -104,7 +114,6 @@ if ($action == 'confirm_execute' && $confirm == "yes" && $user->rights->cron->ex
 
 		if ($result < 0) {
 			setEventMessages($object->error, $object->errors, 'errors');
-			$action = '';
 		} else {
 			$res = $object->reprogram_jobs($user->login, $now);
 			if ($res > 0) {
@@ -113,17 +122,16 @@ if ($action == 'confirm_execute' && $confirm == "yes" && $user->rights->cron->ex
 				} else {
 					setEventMessages($langs->trans("JobFinished"), null, 'mesgs');
 				}
-				$action = '';
 			} else {
 				setEventMessages($object->error, $object->errors, 'errors');
-				$action = '';
 			}
 		}
 	}
+	$action = '';
 }
 
 
-if ($action == 'add') {
+if ($action == 'add' && $permissiontoadd) {
 	$object->jobtype = GETPOST('jobtype');
 	$object->label = GETPOST('label');
 	$object->command = GETPOST('command');
@@ -134,19 +142,21 @@ if ($action == 'add') {
 	$object->md5params = GETPOST('md5params');
 	$object->module_name = GETPOST('module_name');
 	$object->note_private = GETPOST('note', 'restricthtml');
-	$object->datestart = dol_mktime(GETPOST('datestarthour', 'int'), GETPOST('datestartmin', 'int'), 0, GETPOST('datestartmonth', 'int'), GETPOST('datestartday', 'int'), GETPOST('datestartyear', 'int'));
-	$object->dateend = dol_mktime(GETPOST('dateendhour', 'int'), GETPOST('dateendmin', 'int'), 0, GETPOST('dateendmonth', 'int'), GETPOST('dateendday', 'int'), GETPOST('dateendyear', 'int'));
-	$object->priority = GETPOST('priority', 'int');
-	$object->datenextrun = dol_mktime(GETPOST('datenextrunhour', 'int'), GETPOST('datenextrunmin', 'int'), 0, GETPOST('datenextrunmonth', 'int'), GETPOST('datenextrunday', 'int'), GETPOST('datenextrunyear', 'int'));
-	$object->unitfrequency = GETPOST('unitfrequency', 'int');
-	$object->frequency = GETPOST('nbfrequency', 'int');
-	$object->maxrun = GETPOST('maxrun', 'int');
+	$object->datestart = dol_mktime(GETPOSTINT('datestarthour'), GETPOSTINT('datestartmin'), 0, GETPOSTINT('datestartmonth'), GETPOSTINT('datestartday'), GETPOSTINT('datestartyear'));
+	$object->dateend = dol_mktime(GETPOSTINT('dateendhour'), GETPOSTINT('dateendmin'), 0, GETPOSTINT('dateendmonth'), GETPOSTINT('dateendday'), GETPOSTINT('dateendyear'));
+	$object->priority = GETPOSTINT('priority');
+	$object->datenextrun = dol_mktime(GETPOSTINT('datenextrunhour'), GETPOSTINT('datenextrunmin'), 0, GETPOSTINT('datenextrunmonth'), GETPOSTINT('datenextrunday'), GETPOSTINT('datenextrunyear'));
+	$object->unitfrequency = GETPOST('unitfrequency', 'alpha');
+	$object->frequency = GETPOSTINT('nbfrequency');
+	$object->maxrun = GETPOSTINT('maxrun');
 	$object->email_alert = GETPOST('email_alert');
-
+	$object->status = 0;
+	$object->processing = 0;
+	$object->lastresult = '';
 	// Add cron task
 	$result = $object->create($user);
 
-	// test du Resultat de la requete
+	// Test request result
 	if ($result < 0) {
 		setEventMessages($object->error, $object->errors, 'errors');
 		$action = 'create';
@@ -157,7 +167,7 @@ if ($action == 'add') {
 }
 
 // Save parameters
-if ($action == 'update') {
+if ($action == 'update' && $permissiontoadd) {
 	$object->id = $id;
 	$object->jobtype = GETPOST('jobtype');
 	$object->label = GETPOST('label');
@@ -169,19 +179,19 @@ if ($action == 'update') {
 	$object->md5params = GETPOST('md5params');
 	$object->module_name = GETPOST('module_name');
 	$object->note_private = GETPOST('note', 'restricthtml');
-	$object->datestart = dol_mktime(GETPOST('datestarthour', 'int'), GETPOST('datestartmin', 'int'), 0, GETPOST('datestartmonth', 'int'), GETPOST('datestartday', 'int'), GETPOST('datestartyear', 'int'));
-	$object->dateend = dol_mktime(GETPOST('dateendhour', 'int'), GETPOST('dateendmin', 'int'), 0, GETPOST('dateendmonth', 'int'), GETPOST('dateendday', 'int'), GETPOST('dateendyear', 'int'));
-	$object->priority = GETPOST('priority', 'int');
-	$object->datenextrun = dol_mktime(GETPOST('datenextrunhour', 'int'), GETPOST('datenextrunmin', 'int'), 0, GETPOST('datenextrunmonth', 'int'), GETPOST('datenextrunday', 'int'), GETPOST('datenextrunyear', 'int'));
-	$object->unitfrequency = GETPOST('unitfrequency', 'int');
-	$object->frequency = GETPOST('nbfrequency', 'int');
-	$object->maxrun = GETPOST('maxrun', 'int');
+	$object->datestart = dol_mktime(GETPOSTINT('datestarthour'), GETPOSTINT('datestartmin'), 0, GETPOSTINT('datestartmonth'), GETPOSTINT('datestartday'), GETPOSTINT('datestartyear'));
+	$object->dateend = dol_mktime(GETPOSTINT('dateendhour'), GETPOSTINT('dateendmin'), 0, GETPOSTINT('dateendmonth'), GETPOSTINT('dateendday'), GETPOSTINT('dateendyear'));
+	$object->priority = GETPOSTINT('priority');
+	$object->datenextrun = dol_mktime(GETPOSTINT('datenextrunhour'), GETPOSTINT('datenextrunmin'), 0, GETPOSTINT('datenextrunmonth'), GETPOSTINT('datenextrunday'), GETPOSTINT('datenextrunyear'));
+	$object->unitfrequency = GETPOST('unitfrequency', 'alpha');
+	$object->frequency = GETPOSTINT('nbfrequency');
+	$object->maxrun = GETPOSTINT('maxrun');
 	$object->email_alert = GETPOST('email_alert');
 
 	// Add cron task
 	$result = $object->update($user);
 
-	// test du Resultat de la requete
+	// Test request result
 	if ($result < 0) {
 		setEventMessages($object->error, $object->errors, 'errors');
 		$action = 'edit';
@@ -191,13 +201,13 @@ if ($action == 'update') {
 	}
 }
 
-if ($action == 'activate') {
+if ($action == 'activate' && $permissiontoadd) {
 	$object->status = 1;
 
 	// Add cron task
 	$result = $object->update($user);
 
-	// test du Resultat de la requete
+	// Test request result
 	if ($result < 0) {
 		setEventMessages($object->error, $object->errors, 'errors');
 		$action = 'edit';
@@ -207,14 +217,14 @@ if ($action == 'activate') {
 	}
 }
 
-if ($action == 'inactive') {
+if ($action == 'inactive' && $permissiontoadd) {
 	$object->status = 0;
 	$object->processing = 0;
 
 	// Add cron task
 	$result = $object->update($user);
 
-	// test du Resultat de la requete
+	// Test request result
 	if ($result < 0) {
 		setEventMessages($object->error, $object->errors, 'errors');
 		$action = 'edit';
@@ -225,11 +235,11 @@ if ($action == 'inactive') {
 }
 
 // Action clone object
-if ($action == 'confirm_clone' && $confirm == 'yes' && !empty($permissiontoadd)) {
-	if (1 == 0 && !GETPOST('clone_content') && !GETPOST('clone_receivers')) {
+if ($action == 'confirm_clone' && $confirm == 'yes' && $permissiontoadd) {
+	if (1 == 0 && !GETPOST('clone_content') && !GETPOST('clone_receivers')) {  // @phan-suppress-current-line PhanPluginBothLiteralsBinaryOp
 		setEventMessages($langs->trans("NoCloneOptionsSpecified"), null, 'errors');
 	} else {
-		$objectutil = dol_clone($object, 1); // To avoid to denaturate loaded object when setting some properties for clone or if createFromClone modifies the object. We use native clone to keep this->db valid.
+		$objectutil = dol_clone($object, 1); // We clone to avoid to denaturate loaded object when setting some properties for clone or if createFromClone modifies the object. We use the native clone to keep this->db valid.
 
 		$result = $objectutil->createFromClone($user, (($object->id > 0) ? $object->id : $id));
 		if (is_object($result) || $result > 0) {
@@ -249,7 +259,6 @@ if ($action == 'confirm_clone' && $confirm == 'yes' && !empty($permissiontoadd))
 }
 
 
-
 /*
  * View
  */
@@ -333,7 +342,7 @@ if (($action == "create") || ($action == "edit")) {
 	if ($action == "edit") {
 		print dol_get_fiche_head($head, 'card', $langs->trans("CronTask"), 0, 'cron');
 	} else {
-		print dol_get_fiche_head('');
+		print dol_get_fiche_head([]);
 	}
 
 	print '<table class="border centpercent">';
@@ -429,7 +438,7 @@ if (($action == "create") || ($action == "edit")) {
 	print '<tr><td class="fieldrequired">';
 	print $langs->trans('CronEvery')."</td>";
 	print "<td>";
-	print '<select name="nbfrequency">';
+	print '<select name="nbfrequency" id="nbfrequency" class="width50 maxwidth50imp">';
 	for ($i = 1; $i <= 60; $i++) {
 		if ($object->frequency == $i) {
 			print "<option value='".$i."' selected>".$i."</option>";
@@ -438,7 +447,8 @@ if (($action == "create") || ($action == "edit")) {
 		}
 	}
 	print "</select>";
-	$input = " <input type=\"radio\" name=\"unitfrequency\" value=\"60\" id=\"frequency_minute\" ";
+	print ajax_combobox('nbfrequency');
+	$input = " &nbsp;<input type=\"radio\" name=\"unitfrequency\" value=\"60\" id=\"frequency_minute\" ";
 	if ($object->unitfrequency == "60") {
 		$input .= ' checked />';
 	} else {
@@ -447,7 +457,7 @@ if (($action == "create") || ($action == "edit")) {
 	$input .= "<label for=\"frequency_minute\">".$langs->trans('Minutes')."</label>";
 	print $input;
 
-	$input = " <input type=\"radio\" name=\"unitfrequency\" value=\"3600\" id=\"frequency_heures\" ";
+	$input = " &nbsp;<input type=\"radio\" name=\"unitfrequency\" value=\"3600\" id=\"frequency_heures\" ";
 	if ($object->unitfrequency == "3600") {
 		$input .= ' checked />';
 	} else {
@@ -456,7 +466,7 @@ if (($action == "create") || ($action == "edit")) {
 	$input .= "<label for=\"frequency_heures\">".$langs->trans('Hours')."</label>";
 	print $input;
 
-	$input = " <input type=\"radio\" name=\"unitfrequency\" value=\"86400\" id=\"frequency_jours\" ";
+	$input = " &nbsp;<input type=\"radio\" name=\"unitfrequency\" value=\"86400\" id=\"frequency_jours\" ";
 	if ($object->unitfrequency == "86400") {
 		$input .= ' checked />';
 	} else {
@@ -465,7 +475,7 @@ if (($action == "create") || ($action == "edit")) {
 	$input .= "<label for=\"frequency_jours\">".$langs->trans('Days')."</label>";
 	print $input;
 
-	$input = " <input type=\"radio\" name=\"unitfrequency\" value=\"604800\" id=\"frequency_semaine\" ";
+	$input = " &nbsp;<input type=\"radio\" name=\"unitfrequency\" value=\"604800\" id=\"frequency_semaine\" ";
 	if ($object->unitfrequency == "604800") {
 		$input .= ' checked />';
 	} else {
@@ -474,13 +484,13 @@ if (($action == "create") || ($action == "edit")) {
 	$input .= "<label for=\"frequency_semaine\">".$langs->trans('Weeks')."</label>";
 	print $input;
 
-	$input = " <input type=\"radio\" name=\"unitfrequency\" value=\"2678400\" id=\"frequency_month\" ";
+	$input = " &nbsp;<input type=\"radio\" name=\"unitfrequency\" value=\"2678400\" id=\"frequency_month\" ";
 	if ($object->unitfrequency == "2678400") {
 		$input .= ' checked />';
 	} else {
 		$input .= ' />';
 	}
-	$input .= "<label for=\"frequency_month\">".$langs->trans('Monthly')."</label>";
+	$input .= '<label for="frequency_month">'.$langs->trans('Months')."</label>";
 	print $input;
 
 	print "</td>";
@@ -488,10 +498,23 @@ if (($action == "create") || ($action == "edit")) {
 	print "</td>";
 	print "</tr>\n";
 
+	// Priority
+	print "<tr><td>";
+	print $langs->trans('CronPriority')."</td>";
+	$priority = 0;
+	if (!empty($object->priority)) {
+		$priority = $object->priority;
+	}
+	print '<td><input type="text" class="width50" name="priority" value="'.$priority.'" /> ';
+	print "</td>";
+	print "<td>";
+	print "</td>";
+	print "</tr>\n";
+
 	print "<tr><td>";
 	print $langs->trans('CronDtStart')."</td><td>";
 	if (!empty($object->datestart)) {
-		print $form->selectDate($object->datestart, 'datestart', 1, 1, '', "cronform");
+		print $form->selectDate($object->datestart, 'datestart', 1, 1, 0, "cronform");
 	} else {
 		print $form->selectDate(-1, 'datestart', 1, 1, 1, "cronform");
 	}
@@ -503,7 +526,7 @@ if (($action == "create") || ($action == "edit")) {
 	print "<tr><td>";
 	print $langs->trans('CronDtEnd')."</td><td>";
 	if (!empty($object->dateend)) {
-		print $form->selectDate($object->dateend, 'dateend', 1, 1, '', "cronform");
+		print $form->selectDate($object->dateend, 'dateend', 1, 1, 0, "cronform");
 	} else {
 		print $form->selectDate(-1, 'dateend', 1, 1, 1, "cronform");
 	}
@@ -512,18 +535,6 @@ if (($action == "create") || ($action == "edit")) {
 	print "</td>";
 	print "</tr>\n";
 
-	print "<tr><td>";
-	print $langs->trans('CronPriority')."</td>";
-	$priority = 0;
-	if (!empty($object->priority)) {
-		$priority = $object->priority;
-	}
-	print '<td><input type="text" class="width50" name="priority" value="'.$priority.'" /> ';
-	print "</td>";
-	print "<td>";
-	print "</td>";
-	print "</tr>\n";
-
 	print '<tr><td>';
 	$maxrun = '';
 	if (!empty($object->maxrun)) {
@@ -541,9 +552,9 @@ if (($action == "create") || ($action == "edit")) {
 	//print ' ('.$langs->trans('CronFrom').')';
 	print "</td><td>";
 	if (!empty($object->datenextrun)) {
-		print $form->selectDate($object->datenextrun, 'datenextrun', 1, 1, '', "cronform");
+		print $form->selectDate($object->datenextrun, 'datenextrun', 1, 1, 0, "cronform");
 	} else {
-		print $form->selectDate(-1, 'datenextrun', 1, 1, '', "cronform", 1, 1);
+		print $form->selectDate(-1, 'datenextrun', 1, 1, 0, "cronform", 1, 1);
 	}
 	print "</td>";
 	print "<td>";
@@ -558,9 +569,7 @@ if (($action == "create") || ($action == "edit")) {
 
 	print "</form>\n";
 } else {
-	/*
-	 * view card
-	 */
+	// view card
 	$now = dol_now();
 
 	print dol_get_fiche_head($head, 'card', $langs->trans("CronTask"), -1, 'cron');
@@ -592,7 +601,7 @@ if (($action == "create") || ($action == "edit")) {
 	print "<td>".$langs->trans($object->label);
 	print "</td></tr>";*/
 
-	print '<tr><td class="titlefield">';
+	print '<tr><td class="titlefieldmiddle">';
 	print $langs->trans('CronType')."</td><td>";
 	print $formCron->select_typejob('jobtype', $object->jobtype, 1);
 	print "</td></tr>";
@@ -630,7 +639,7 @@ if (($action == "create") || ($action == "edit")) {
 	print '<tr><td>';
 	print $langs->trans('CronNote')."</td><td>";
 	if (!is_null($object->note_private) && $object->note_private != '') {
-		print $langs->trans($object->note_private);
+		print '<div class="small lineheightsmall">'.$langs->trans($object->note_private).'</div>';
 	}
 	print "</td></tr>";
 
@@ -659,7 +668,7 @@ if (($action == "create") || ($action == "edit")) {
 	print '<div class="underbanner clearboth"></div>';
 	print '<table class="border centpercent tableforfield">';
 
-	print '<tr><td class="titlefield">';
+	print '<tr><td class="titlefieldmiddle">';
 	print $langs->trans('CronEvery')."</td>";
 	print "<td>";
 	if ($object->unitfrequency == "60") {
@@ -675,10 +684,16 @@ if (($action == "create") || ($action == "edit")) {
 		print $langs->trans('CronEach')." ".($object->frequency)." ".$langs->trans('Weeks');
 	}
 	if ($object->unitfrequency == "2678400") {
-		print $langs->trans('CronEach')." ".($object->frequency)." ".$langs->trans('Month');
+		print $langs->trans('CronEach')." ".($object->frequency)." ".$langs->trans('Months');
 	}
 	print "</td></tr>";
 
+	// Priority
+	print "<tr><td>";
+	print $langs->trans('CronPriority')."</td>";
+	print "<td>".$object->priority;
+	print "</td></tr>";
+
 	print '<tr><td>';
 	print $langs->trans('CronDtStart')."</td><td>";
 	if (!empty($object->datestart)) {
@@ -693,11 +708,6 @@ if (($action == "create") || ($action == "edit")) {
 	}
 	print "</td></tr>";
 
-	print "<tr><td>";
-	print $langs->trans('CronPriority')."</td>";
-	print "<td>".$object->priority;
-	print "</td></tr>";
-
 	print "<tr><td>";
 	print $langs->trans('CronMaxRun')."</td>";
 	print "<td>";
@@ -715,11 +725,12 @@ if (($action == "create") || ($action == "edit")) {
 	print ' ('.$langs->trans('CronFrom').')';
 	print "</td><td>";
 	if (!$object->status) {
+		print img_picto('', 'object_calendarday').' <span class="opacitymedium strikefordisabled">'.$form->textwithpicto(dol_print_date($object->datenextrun, 'dayhoursec'), $langs->trans("CurrentTimeZone")).'</span> ';
 		print $langs->trans("Disabled");
 	} elseif (!empty($object->datenextrun)) {
 		print img_picto('', 'object_calendarday').' '.$form->textwithpicto(dol_print_date($object->datenextrun, 'dayhoursec'), $langs->trans("CurrentTimeZone"));
 	} else {
-		print $langs->trans('CronNone');
+		print '<span class="opacitymedium">'.$langs->trans('CronNone').'</span>';
 	}
 	if ($object->status == Cronjob::STATUS_ENABLED) {
 		if ($object->maxrun && $object->nbrun >= $object->maxrun) {
@@ -739,12 +750,12 @@ if (($action == "create") || ($action == "edit")) {
 	print '<div class="underbanner clearboth"></div>';
 	print '<table class="border centpercent tableforfield">';
 
-	print '<tr><td class="titlefield">';
+	print '<tr><td class="titlefieldmiddle">';
 	print $langs->trans('CronDtLastLaunch')."</td><td>";
 	if (!empty($object->datelastrun)) {
 		print $form->textwithpicto(dol_print_date($object->datelastrun, 'dayhoursec'), $langs->trans("CurrentTimeZone"));
 	} else {
-		print $langs->trans('CronNone');
+		print '<span class="opacitymedium">'.$langs->trans('CronNotYetRan').'</span>';
 	}
 	print "</td></tr>";
 
@@ -754,7 +765,7 @@ if (($action == "create") || ($action == "edit")) {
 		print $form->textwithpicto(dol_print_date($object->datelastresult, 'dayhoursec'), $langs->trans("CurrentTimeZone"));
 	} else {
 		if (empty($object->datelastrun)) {
-			print $langs->trans('CronNone');
+			print '<span class="opacitymedium">'.$langs->trans('CronNotYetRan').'</span>';
 		} else {
 			// In progress
 		}
@@ -774,7 +785,7 @@ if (($action == "create") || ($action == "edit")) {
 
 	print '<tr><td>';
 	print $langs->trans('CronLastOutput')."</td><td>";
-	print '<span class="small">'.nl2br($object->lastoutput).'</span>';
+	print '<span class="small">'.(!empty($object->lastoutput) ? nl2br($object->lastoutput) : '').'</span>';
 	print "</td></tr>";
 
 	print '</table>';
@@ -788,21 +799,21 @@ if (($action == "create") || ($action == "edit")) {
 
 
 	print "\n\n".'<div class="tabsAction">'."\n";
-	if (!$user->rights->cron->create) {
+	if (!$user->hasRight('cron', 'create')) {
 		print '<a class="butActionRefused classfortooltip" href="#" title="'.dol_escape_htmltag($langs->transnoentitiesnoconv("NotEnoughPermissions")).'">'.$langs->trans("Edit").'</a>';
 	} else {
 		print '<a class="butAction" href="'.$_SERVER['PHP_SELF'].'?action=edit&token='.newToken().'&id='.$object->id.'">'.$langs->trans("Edit").'</a>';
 	}
 
-	if ((empty($user->rights->cron->execute))) {
+	if ((!$user->hasRight('cron', 'execute'))) {
 		print '<a class="butActionRefused classfortooltip" href="#" title="'.dol_escape_htmltag($langs->transnoentitiesnoconv("NotEnoughPermissions")).'">'.$langs->trans("CronExecute").'</a>';
 	} elseif (empty($object->status)) {
 		print '<a class="butActionRefused classfortooltip" href="#" title="'.dol_escape_htmltag($langs->transnoentitiesnoconv("JobDisabled")).'">'.$langs->trans("CronExecute").'</a>';
 	} else {
-		print '<a class="butAction" href="'.$_SERVER['PHP_SELF'].'?action=execute&token='.newToken().'&id='.$object->id.(empty($conf->global->CRON_KEY) ? '' : '&securitykey='.$conf->global->CRON_KEY).'">'.$langs->trans("CronExecute").'</a>';
+		print '<a class="butAction" href="'.$_SERVER['PHP_SELF'].'?action=execute&token='.newToken().'&id='.$object->id.(!getDolGlobalString('CRON_KEY') ? '' : '&securitykey='.urlencode(getDolGlobalString('CRON_KEY'))).'">'.$langs->trans("CronExecute").'</a>';
 	}
 
-	if (!$user->rights->cron->create) {
+	if (!$user->hasRight('cron', 'create')) {
 		print '<a class="butActionRefused classfortooltip" href="#" title="'.dol_escape_htmltag($langs->transnoentitiesnoconv("NotEnoughPermissions")).'">'.$langs->trans("CronStatusActiveBtn").'/'.$langs->trans("CronStatusInactiveBtn").'</a>';
 	} else {
 		print '<a class="butAction" href="'.$_SERVER['PHP_SELF'].'?action=clone&token='.newToken().'&id='.$object->id.'">'.$langs->trans("ToClone").'</a>';
@@ -814,7 +825,7 @@ if (($action == "create") || ($action == "edit")) {
 		}
 	}
 
-	if (!$user->rights->cron->delete) {
+	if (!$user->hasRight('cron', 'delete')) {
 		print '<a class="butActionDeleteRefused" href="#" title="'.dol_escape_htmltag($langs->transnoentitiesnoconv("NotEnoughPermissions")).'">'.$langs->trans("Delete").'</a>';
 	} else {
 		print '<a class="butActionDelete" href="'.$_SERVER['PHP_SELF'].'?action=delete&token='.newToken().'&id='.$object->id.'">'.$langs->trans("Delete").'</a>';
